@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/compile.c,v 1.76 2001/04/09 04:28:24 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/compile.c,v 1.83 2001/08/14 08:34:13 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -21,6 +21,7 @@ ObjMark (void *object)
     InstPtr inst;
     int	    i;
 
+    MemReference (obj->nonLocal);
     inst = ObjCode (obj, 0);
     for (i = 0; i < obj->used; i++, inst++)
     {
@@ -69,7 +70,8 @@ NewObj (int size)
     obj = ALLOCATE (&ObjType, sizeof (Obj) + size * sizeof (Inst));
     obj->size = size;
     obj->used = 0;
-    obj->state = 0;
+    obj->error = False;
+    obj->nonLocal = 0;
     RETURN (obj);
 }
 
@@ -86,7 +88,8 @@ AddInst (ObjPtr obj)
 	nobj = NewObj (obj->size + OBJ_INCR);
 	memcpy (ObjCode (nobj, 0), ObjCode (obj, 0), obj->used * sizeof (Inst));
 	nobj->used = obj->used;
-	nobj->state = obj->state;
+	nobj->error = obj->error;
+	nobj->nonLocal = obj->nonLocal;
 	obj = nobj;
     }
     obj->used++;
@@ -113,7 +116,7 @@ AddInst (ObjPtr obj)
 
 typedef enum _tail { TailNever, TailVoid, TailAlways } Tail;
 
-ObjPtr	CompileLvalue (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code, Bool createIfNecessary, Bool assign);
+ObjPtr	CompileLvalue (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code, Bool createIfNecessary, Bool assign, Bool initialize);
 ObjPtr	CompileBinary (ObjPtr obj, ExprPtr expr, OpCode opCode, ExprPtr stat, CodePtr code);
 ObjPtr	CompileUnary (ObjPtr obj, ExprPtr expr, OpCode opCode, ExprPtr stat, CodePtr code);
 ObjPtr	CompileAssign (ObjPtr obj, ExprPtr expr, OpCode opCode, ExprPtr stat, CodePtr code);
@@ -136,7 +139,7 @@ CompileStorage (ObjPtr obj, ExprPtr stat, SymbolPtr symbol, CodePtr code)
     ENTER ();
     
     if (!symbol)
-	obj->state |= OBJ_STATE_ERROR;
+	obj->error = True;
     /*
      * For symbols hanging from a frame (statics, locals and args),
      * locate the frame and set their element value
@@ -253,7 +256,7 @@ CompileError (ObjPtr obj, ExprPtr stat, char *s, ...)
     FileVPrintf (FileStderr, s, args);
     va_end (args);
     FilePrintf (FileStderr, "\n");
-    obj->state |= OBJ_STATE_ERROR;
+    obj->error = True;
 }
 
 /*
@@ -262,7 +265,7 @@ CompileError (ObjPtr obj, ExprPtr stat, char *s, ...)
  */
 ObjPtr
 CompileLvalue (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code,
-		Bool createIfNecessary, Bool assign)
+		Bool createIfNecessary, Bool assign, Bool initialize)
 {
     ENTER ();
     InstPtr	inst;
@@ -279,7 +282,7 @@ CompileLvalue (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code,
 	    expr->base.type = typesPoly;
 	    break;
 	}
-	if (!ClassStorage (s->symbol.class))
+	if (!ClassStorage (s->symbol.class) || (!initialize && s->symbol.class == class_const))
 	{
 	    CompileError (obj, stat, "Invalid use of %C \"%A\"",
 			  s->symbol.class, expr->atom.atom);
@@ -300,7 +303,7 @@ CompileLvalue (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code,
 	expr->base.type = s->symbol.type;
 	break;
     case COLONCOLON:
-	obj = CompileLvalue (obj, expr->tree.right, stat, code, False, assign);
+	obj = CompileLvalue (obj, expr->tree.right, stat, code, False, assign, initialize);
 	expr->base.type = expr->tree.right->base.type;
         break;
     case DOT:
@@ -459,7 +462,7 @@ CompileAssign (ObjPtr obj, ExprPtr expr, OpCode opCode, ExprPtr stat, CodePtr co
     obj = _CompileExpr (obj, expr->tree.right, True, stat, code);
     SetPush (obj);
     obj = CompileLvalue (obj, expr->tree.left, stat, code, opCode == OpAssign, 
-			 opCode == OpAssign || opCode == OpInitialize);
+			 opCode == OpAssign || opCode == OpInitialize, opCode == OpInitialize);
     expr->base.type = TypeCombineBinary (expr->tree.left->base.type,
 					 expr->base.tag,
 					 expr->tree.right->base.type);
@@ -474,18 +477,6 @@ CompileAssign (ObjPtr obj, ExprPtr expr, OpCode opCode, ExprPtr stat, CodePtr co
     RETURN (obj);
 }
 
-/*
- * For typechecking, access arguments from left to right
- */
-
-static ExprPtr
-CompileGetArg (ExprPtr arg, int i)
-{
-    while (arg && --i)
-	arg = arg->tree.right;
-    return arg;
-}
-
 static ObjPtr
 CompileArgs (ObjPtr obj, int *argcp, ExprPtr arg, ExprPtr stat, CodePtr code)
 {
@@ -533,9 +524,9 @@ CompileTypecheckArgs (ObjPtr	obj,
     if (func_type->base.tag == types_func)
     {
 	argt = func_type->func.args;
+	arg = args;
 	i = 0;
-	while ((arg = CompileGetArg (args, argc - i)) || 
-	       (argt && !argt->varargs))
+	while (arg || (argt && !argt->varargs))
 	{
 	    if (!argt)
 	    {
@@ -555,7 +546,8 @@ CompileTypecheckArgs (ObjPtr	obj,
 			      argt->type, arg->tree.left->base.type, i);
 		ret = False;
 	    }
-	    i++;
+	    if (arg)
+		arg = arg->tree.right;
 	    if (!argt->varargs)
 		argt = argt->next;
 	}
@@ -564,6 +556,30 @@ CompileTypecheckArgs (ObjPtr	obj,
     return ret;
 }
 
+
+static void
+MarkNonLocal (void *object)
+{
+    NonLocal	*nl = object;
+
+    MemReference (nl->prev);
+}
+
+DataType    NonLocalType = { MarkNonLocal, 0 };
+
+static NonLocal *
+NewNonLocal (NonLocal *prev, NonLocalKind kind, int target)
+{
+    ENTER();
+    NonLocal	*nl;
+
+    nl = ALLOCATE (&NonLocalType, sizeof (NonLocal));
+    nl->prev = prev;
+    nl->kind = kind;
+    nl->target = target;
+    RETURN (nl);
+}
+
 /*
  * Compile a function call --
  * 
@@ -632,7 +648,17 @@ CompileRaise (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code)
     sym = name->atom.symbol;
 
     if (!sym)
+    {
+	CompileError (obj, stat, "No exception '%A' in scope",
+		      name->atom.atom);
 	RETURN (obj);
+    }
+    if (sym->symbol.class != class_exception)
+    {
+	CompileError (obj, stat, "'%A' is not an exception",
+		      name->atom.atom);
+	RETURN (obj);
+    }
     obj = CompileArgs (obj, &argc, expr->tree.right, stat, code);
     if (!CompileTypecheckArgs (obj, sym->symbol.type, expr->tree.right, argc, stat))
 	RETURN(obj);
@@ -670,7 +696,6 @@ CompileTwixt (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code)
     ENTER ();
     int	    enter_inst, twixt_inst, twixt_body_inst,
 	    enter_done_inst, leave_done_inst;
-    int     state;
     InstPtr inst;
     
     enter_inst = obj->used;
@@ -690,11 +715,11 @@ CompileTwixt (ObjPtr obj, ExprPtr expr, ExprPtr stat, CodePtr code)
 
     /* Compile the body */
     twixt_body_inst = obj->used;
-    state = obj->state;
-    obj->state |= OBJ_STATE_SWITCH;
+    obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalTwixt,
+				 NON_LOCAL_BREAK);
     obj = _CompileStat (obj, expr->tree.right->tree.left, False, code);
+    obj->nonLocal = obj->nonLocal->prev;
     CompilePatchLoop (obj, twixt_body_inst, -1);
-    obj->state = (obj->state & ~OBJ_STATE_SWITCH) | (state & OBJ_STATE_SWITCH);
     BuildInst (obj, OpTwixtDone, inst, stat);
     
     /* finish the twixt instruction */
@@ -1013,7 +1038,7 @@ CompileCompositeImplicitInit (Types *type)
     
     switch (type->base.tag) {
     case types_array:
-	if (type->array.dimensions->tree.left)
+	if (type->array.dimensions && type->array.dimensions->tree.left)
 	{
 	    sub = type->array.type;
 	    init = CompileCompositeImplicitInit (sub);
@@ -1207,8 +1232,12 @@ CompileCatch (ObjPtr obj, ExprPtr catches, ExprPtr body,
 	inst->catch.offset = obj->used - catch_inst;
 	inst->catch.exception = exception;
     
+	obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalCatch, 0);
+	
 	obj = CompileCatch (obj, catches->tree.right, body, stat, code);
 	
+	obj->nonLocal = obj->nonLocal->prev;
+
 	BuildInst (obj, OpEndCatch, inst, stat);
 	
 	inst = ObjCode (obj, exception_inst);
@@ -1289,7 +1318,13 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	}
 	break;
     case UNION:
-	_CompileExpr (obj, expr->tree.right, True, stat, code);
+	if (expr->tree.right)
+	    _CompileExpr (obj, expr->tree.right, True, stat, code);
+	else
+	{
+	    BuildInst (obj, OpConst, inst, stat);
+	    inst->constant.constant = Void;
+	}
 	SetPush (obj);
 	t = TypesCanon (expr->base.type);
 	if (t && t->base.tag == types_union)
@@ -1306,14 +1341,32 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	    }
 	    BuildInst (obj, OpBuildUnion, inst, stat);
 	    inst->structs.structs = st;
-	    if (!TypeCombineBinary (expr->tree.left->base.type,
-				    ASSIGN,
-				    expr->tree.right->base.type))
+	    if (expr->tree.left->base.type == typesEnum)
 	    {
-		CompileError (obj, stat, "Incompatible types '%T', '%T' in union constructor",
-			      expr->tree.left->base.type,
-			      expr->tree.right->base.type);
-		break;
+		if (expr->tree.right)
+		{
+		    CompileError (obj, stat, "enum member '%A' takes no constructor argument",
+				  expr->tree.left->atom.atom);
+		    break;
+		}
+	    }
+	    else
+	    {
+		if (!expr->tree.right)
+		{
+		    CompileError (obj, stat, "union member '%A' requires constructor argument",
+				  expr->tree.left->atom.atom);
+		    break;
+		}
+		if (!TypeCombineBinary (expr->tree.left->base.type,
+					ASSIGN,
+					expr->tree.right->base.type))
+		{
+		    CompileError (obj, stat, "Incompatible types '%T', '%T' in union constructor",
+				  expr->tree.left->base.type,
+				  expr->tree.right->base.type);
+		    break;
+		}
 	    }
 	    BuildInst (obj, OpInitUnion, inst, stat);
 	    inst->atom.atom = expr->tree.left->atom.atom;
@@ -1396,7 +1449,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	{
 	    CompileError (obj, stat, "Type '%T' is not a struct or union containing \"%A\"",
 			  expr->tree.left->base.type,
-			  expr->tree.left->atom.atom);
+			  expr->tree.right->atom.atom);
 	    expr->base.type = typesPoly;
 	    break;
 	}
@@ -1426,7 +1479,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	break;
     case STAR:	    obj = CompileUnary (obj, expr, OpStar, stat, code); break;
     case AMPER:	    
-	obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False);
+	obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False, False);
 	expr->base.type = NewTypesRef (expr->tree.left->base.type);
 	if (!expr->base.type)
 	{
@@ -1443,7 +1496,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
     case INC:	    
 	if (expr->tree.left)
 	{
-	    obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False);
+	    obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False, False);
 	    expr->base.type = TypeCombineBinary (expr->tree.left->base.type,
 						 ASSIGNPLUS,
 						 typesPrim[type_int]);
@@ -1451,7 +1504,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	}
 	else
 	{
-	    obj = CompileLvalue (obj, expr->tree.right, stat, code, False, False);
+	    obj = CompileLvalue (obj, expr->tree.right, stat, code, False, False, False);
 	    expr->base.type = TypeCombineBinary (expr->tree.right->base.type,
 						 ASSIGNPLUS,
 						 typesPrim[type_int]);
@@ -1469,7 +1522,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
     case DEC:
 	if (expr->tree.left)
 	{
-	    obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False);
+	    obj = CompileLvalue (obj, expr->tree.left, stat, code, False, False, False);
 	    expr->base.type = TypeCombineBinary (expr->tree.left->base.type,
 						 ASSIGNMINUS,
 						 typesPrim[type_int]);
@@ -1477,7 +1530,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
 	}
 	else
 	{
-	    obj = CompileLvalue (obj, expr->tree.right, stat, code, False, False);
+	    obj = CompileLvalue (obj, expr->tree.right, stat, code, False, False, False);
 	    expr->base.type = TypeCombineBinary (expr->tree.right->base.type,
 						 ASSIGNMINUS,
 						 typesPrim[type_int]);
@@ -1499,10 +1552,10 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
     case DIV:	    obj = CompileBinary (obj, expr, OpDiv, stat, code); break;
     case MOD:	    obj = CompileBinary (obj, expr, OpMod, stat, code); break;
     case POW:
-	obj = _CompileExpr (obj, expr->tree.right->tree.right, True, stat, code);
-	SetPush (obj);
 	obj = _CompileExpr (obj, expr->tree.right->tree.left, True, stat, code);
 	SetPush (obj);
+	obj = _CompileExpr (obj, expr->tree.right->tree.right, True, stat, code);
+	SetPush (obj);
 	obj = _CompileExpr (obj, expr->tree.left, True, stat, code);
 	expr->base.type = TypeCombineBinary (expr->tree.right->tree.left->base.type,
 					     expr->base.tag,
@@ -1620,7 +1673,7 @@ _CompileExpr (ObjPtr obj, ExprPtr expr, Bool evaluate, ExprPtr stat, CodePtr cod
     case ASSIGNPOW:
 	obj = _CompileExpr (obj, expr->tree.right->tree.right, True, stat, code);
 	SetPush (obj);
-	obj = CompileLvalue (obj, expr->tree.right->tree.left, stat, code, False, False);
+	obj = CompileLvalue (obj, expr->tree.right->tree.left, stat, code, False, False, False);
 	SetPush (obj);
 	obj = _CompileExpr (obj, expr->tree.left, True, stat, code);
 	expr->base.type = TypeCombineBinary (expr->tree.right->tree.left->base.type,
@@ -1709,6 +1762,50 @@ CompilePatchLoop (ObjPtr obj, int start, int continue_offset)
     }
 }
 
+static ObjPtr
+CompileNonLocal (ObjPtr obj, int target, ExprPtr expr)
+{
+    ENTER ();
+    NonLocal	*nl;
+    int		twixt = 0, catch = 0;
+
+    for (nl = obj->nonLocal; nl; nl = nl->prev)
+    {
+	if (nl->target & target)
+	    break;
+	switch (nl->kind) {
+	case NonLocalTwixt:
+	    twixt++;
+	    break;
+	case NonLocalCatch:
+	    catch++;
+	    break;
+	case NonLocalControl:
+	    break;
+	}
+    }
+    if (!nl) 
+    {
+	switch (target) {
+	case NON_LOCAL_BREAK:
+	    CompileError (obj, expr, "break not in loop/switch/twixt");
+	    break;
+	case NON_LOCAL_CONTINUE:
+	    CompileError (obj, expr, "continue not in loop");
+	    break;
+	}
+    }
+    if (twixt || catch)
+    {
+	InstPtr	inst;
+	BuildInst (obj, OpUnwind, inst, expr);
+	inst->base.stat = expr;
+	inst->unwind.twixt = twixt;
+	inst->unwind.catch = catch;
+    }
+    RETURN (obj);
+}
+
 ObjPtr
 _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 {
@@ -1716,7 +1813,6 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
     int		top_inst, continue_inst, test_inst, middle_inst, bottom_inst;
     ExprPtr	c;
     int		ncase, *case_inst, icase;
-    int		state;
     Bool	has_default;
     InstPtr	inst;
     
@@ -1769,10 +1865,10 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	continue_inst = obj->used;
 	obj = _CompileExpr (obj, expr->tree.left, True, expr, code);
 	NewInst (test_inst, obj);
-	state = obj->state;
-	obj->state |= OBJ_STATE_LOOP;
+	obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalControl,
+				     NON_LOCAL_BREAK|NON_LOCAL_CONTINUE);
 	obj = _CompileStat (obj, expr->tree.right, False, code);
-	obj->state = (obj->state & ~OBJ_STATE_LOOP) | (state & OBJ_STATE_LOOP);
+	obj->nonLocal = obj->nonLocal->prev;
 	NewInst (bottom_inst, obj);
 	inst = ObjCode (obj, test_inst);
 	inst->base.opCode = OpWhile;
@@ -1792,10 +1888,10 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	 * +---+
 	 */
 	continue_inst = obj->used;
-	state = obj->state;
-	obj->state |= OBJ_STATE_LOOP;
+	obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalControl,
+				     NON_LOCAL_BREAK|NON_LOCAL_CONTINUE);
 	obj = _CompileStat (obj, expr->tree.left, False, code);
-	obj->state = (obj->state & ~OBJ_STATE_LOOP) | (state & OBJ_STATE_LOOP);
+	obj->nonLocal = obj->nonLocal->prev;
 	obj = _CompileExpr (obj, expr->tree.right, True, expr, code);
 	NewInst (test_inst, obj);
 	inst = ObjCode (obj, test_inst);
@@ -1803,7 +1899,6 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	inst->base.stat = expr;
 	inst->branch.offset = continue_inst - test_inst;
 	CompilePatchLoop (obj, continue_inst, continue_inst);
-	obj->state = (obj->state & ~OBJ_STATE_LOOP) | (state & OBJ_STATE_LOOP);
 	break;
     case FOR:
 	/*
@@ -1822,10 +1917,10 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	    obj = _CompileExpr (obj, expr->tree.left->tree.right, True, expr, code);
 	    NewInst (test_inst, obj);
 	}
-	state = obj->state;
-	obj->state |= OBJ_STATE_LOOP;
+	obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalControl,
+				     NON_LOCAL_BREAK|NON_LOCAL_CONTINUE);
 	obj = _CompileStat (obj, expr->tree.right->tree.right, False, code);
-	obj->state = (obj->state & ~OBJ_STATE_LOOP) | (state & OBJ_STATE_LOOP);
+	obj->nonLocal = obj->nonLocal->prev;
 	continue_inst = obj->used;
 	if (expr->tree.right->tree.left)
 	    obj = _CompileExpr (obj, expr->tree.right->tree.left, False, expr, code);
@@ -1887,8 +1982,8 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	/* add default case at the bottom */
     	NewInst (test_inst, obj);
 	top_inst = obj->used;
-	state = obj->state;
-	obj->state |= OBJ_STATE_SWITCH;
+	obj->nonLocal = NewNonLocal (obj->nonLocal, NonLocalControl,
+				     NON_LOCAL_BREAK);
 	/*
 	 * Compile the statements
 	 */
@@ -1935,7 +2030,7 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	    }
 	    c = c->tree.right;
 	}
-	obj->state = (obj->state & ~OBJ_STATE_SWITCH) | (state & OBJ_STATE_SWITCH);
+	obj->nonLocal = obj->nonLocal->prev;
 	if (!has_default)
 	{
 	    inst = ObjCode (obj, test_inst);
@@ -1961,8 +2056,7 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	}
 	break;
     case BREAK:
-	if ((obj->state & (OBJ_STATE_LOOP|OBJ_STATE_SWITCH)) == 0)
-	    CompileError (obj, expr, "break not in loop/switch/twixt");
+	obj = CompileNonLocal (obj, NON_LOCAL_BREAK, expr);
 	NewInst (middle_inst, obj);
 	inst = ObjCode (obj, middle_inst);
 	inst->base.opCode = OpBreak;
@@ -1970,8 +2064,7 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	inst->branch.offset = 0;    /* filled in by PatchLoop */
 	break;
     case CONTINUE:
-	if ((obj->state & OBJ_STATE_LOOP) == 0)
-	    CompileError (obj, expr, "continue not in loop");
+	obj = CompileNonLocal (obj, NON_LOCAL_CONTINUE, expr);
 	NewInst (middle_inst, obj);
 	inst = ObjCode (obj, middle_inst);
 	inst->base.opCode = OpContinue;
@@ -1993,6 +2086,7 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	    else
 	    {
 		obj = _CompileExpr (obj, expr->tree.right, True, expr, code);
+		obj = CompileNonLocal (obj, NON_LOCAL_RETURN, expr);
 		BuildInst (obj, OpReturn, inst, expr);
 		inst->base.stat = expr;
 	    }
@@ -2000,6 +2094,7 @@ _CompileStat (ObjPtr obj, ExprPtr expr, Bool last, CodePtr code)
 	}
 	else
 	{
+	    obj = CompileNonLocal (obj, NON_LOCAL_RETURN, expr);
 	    BuildInst (obj, OpReturnVoid, inst, expr);
 	    inst->base.stat = expr;
 	    expr->base.type = typesPrim[type_void];
@@ -2128,7 +2223,7 @@ CompileFunc (ObjPtr obj, CodePtr code, ExprPtr stat, CodePtr previous)
 	    code->base.argc++;
     }
     code->func.body.obj = CompileFuncCode (code, stat, previous);
-    obj->state |= code->func.body.obj->state & OBJ_STATE_ERROR;
+    obj->error |= code->func.body.obj->error;
     BuildInst (obj, OpObj, inst, stat);
     inst->code.code = code;
     if ((staticInit = code->func.staticInit.obj))
@@ -2142,7 +2237,7 @@ CompileFunc (ObjPtr obj, CodePtr code, ExprPtr stat, CodePtr previous)
 	code->func.staticInit.obj = staticInit;
 	BuildInst (obj, OpStaticInit, inst, stat);
 	BuildInst (obj, OpNoop, inst, stat);
-	obj->state |= (staticInit->state & OBJ_STATE_ERROR);
+	obj->error |= staticInit->error;
     }
     RETURN (obj);
 }
@@ -2171,6 +2266,7 @@ CompileDecl (ObjPtr obj, ExprPtr decls,
     publish = decls->decl.publish;
     switch (class) {
     case class_global:
+    case class_const:
 	/*
 	 * Globals are compiled in the static initializer for
 	 * the outermost enclosing function.
@@ -2232,7 +2328,7 @@ CompileDecl (ObjPtr obj, ExprPtr decls,
 		    code->func.inGlobalInit = True;
 	    }
 	    *initObj = _CompileExpr (*initObj, init, 
-				     True, decls, code);
+				     True, stat, code);
 	    if (code_compile)
 	    {
 		code_compile->func.inStaticInit = False;
@@ -2245,7 +2341,7 @@ CompileDecl (ObjPtr obj, ExprPtr decls,
 	     */
 	    lvalue = NewExprAtom (decl->name, decl->symbol);
 	    *initObj = CompileLvalue (*initObj, lvalue,
-				       decls, code, False, True);
+				       decls, code, False, True, True);
 	    if (!TypeCombineBinary (lvalue->base.type,
 				    ASSIGN,
 				    init->base.type))
@@ -2336,6 +2432,7 @@ char *OpNames[] = {
     "OpTwixtDone",
     "OpEnterDone",
     "OpLeaveDone",
+    "OpUnwind",
     /*
      * Expr op codes
      */
@@ -2484,6 +2581,10 @@ InstDump (InstPtr inst, int indent, int i, int *branch, int maxbranch)
 	FilePrintf (FileStdout, "enter %d leave %d",
 		    inst->twixt.enter, inst->twixt.leave);
 	break;
+    case OpUnwind:
+	FilePrintf (FileStdout, "twixt %d catch %d",
+		    inst->unwind.twixt, inst->unwind.catch);
+	break;
     case OpName:
     case OpNameRef:
     case OpNameRefStore:
@@ -2552,6 +2653,7 @@ ObjDump (ObjPtr obj, int indent)
 {
     int	    i, j;
     InstPtr inst;
+    ExprPtr stat;
     int	    *branch;
     int	    b;
 
@@ -2573,9 +2675,16 @@ ObjDump (ObjPtr obj, int indent)
 	}
     }
     b = 0;
+    stat = 0;
     for (i = 0; i < obj->used; i++)
     {
 	inst = ObjCode(obj, i);
+	if (inst->base.stat != stat)
+	{
+	    FilePrintf (FileStdout, "                                     ");
+	    PrettyStat (FileStdout, inst->base.stat, False);
+	    stat = inst->base.stat;
+	}
 	if (branch[i])
 	    FilePrintf (FileStdout, "L%d:\n", branch[i]);
 	InstDump (inst, indent, i, branch, obj->used);
