@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/execute.c,v 1.50 2001/04/09 04:28:24 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/execute.c,v 1.54 2001/08/14 08:34:13 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -22,10 +22,12 @@ Bool	signalSuspend;	    /* current thread is suspending */
 
 static ThreadState ThreadStep (Value thread);
 
+#define Arg(n)  Stack((argc - 1) - ((n)+off))
+
 static FramePtr
 BuildFrame (Value thread, Value func, Bool staticInit, Bool tail,
 	    Bool varargs, int nformal, int off,
-	    int nargs, InstPtr savePc)
+	    int argc, InstPtr savePc)
 {
     ENTER ();
     CodePtr	    code = func->func.code;
@@ -41,16 +43,16 @@ BuildFrame (Value thread, Value func, Bool staticInit, Bool tail,
 		      body->dynamics,
 		      func->func.statics);
     for (fe = 0; fe < nformal; fe++)
-	BoxValueSet (frame->frame, fe, Copy (Stack(fe+off)));
+	BoxValueSet (frame->frame, fe, Copy (Arg(fe)));
     if (varargs)
     {
-	int	extra = nargs - nformal;
+	int	extra = argc - nformal;
 	Value	array;
 	
 	array = NewArray (True, typesPoly, 1, &extra);
 	BoxValueSet (frame->frame, fe, array);
-	for (; fe < nargs; fe++)
-	    BoxValueSet (array->array.values, fe-nformal, Stack(fe+off));
+	for (; fe < argc; fe++)
+	    BoxValueSet (array->array.values, fe-nformal, Copy (Arg(fe)));
     }
     if (tail)
     {
@@ -102,11 +104,11 @@ ThreadCall (Value thread, Bool tail, InstPtr *next, int *stack)
 				    2, NewInt (argc), NewInt(code->base.argc));
 	    RETURN (value);
 	}
-	if (!TypeCompatibleAssign (argt->type, Stack(fe+off)))
+	if (!TypeCompatibleAssign (argt->type, Arg(fe)))
 	{
 	    RaiseStandardException (exception_invalid_argument, 
 				    "Incompatible argument",
-				    2, NewInt (fe), Stack(fe+off));
+				    2, NewInt (fe), Arg(fe));
 	    RETURN (value);
 	}
 	fe++;
@@ -124,8 +126,6 @@ ThreadCall (Value thread, Bool tail, InstPtr *next, int *stack)
 	if (code->base.varargs)
 	    formal = -1;
 
-#define Arg(n)  Stack((n)+off)
-
 	if (code->builtin.needsNext) 
 	{
 	    /*
@@ -238,10 +238,10 @@ ThreadStaticInit (Value thread, InstPtr *next)
 }
 
 static void
-ThreadAssign (Value ref, Value v, Bool copy)
+ThreadAssign (Value ref, Value v, Bool initialize)
 {
     ENTER ();
-    if (RefConstant(ref))
+    if (RefConstant(ref) && !initialize)
     {
 	RaiseStandardException (exception_readonly_box,
 				"Attempted assignment to constant box",
@@ -263,8 +263,7 @@ ThreadAssign (Value ref, Value v, Bool copy)
     {
 	if (!v)
 	    abort ();
-	if (copy)
-	    v = Copy (v);
+	v = Copy (v);
 	if (!aborting)
 	{
 	    complete = True;
@@ -454,6 +453,34 @@ ThreadTwixt (Value thread, int enterOffset, int leaveOffset)
     EXIT ();
 }
 
+static void
+ThreadUnwind (Value thread, int twixts, int catches, InstPtr *next)
+{
+    ENTER ();
+    Value	continuation;
+    CatchPtr	catch;
+    TwixtPtr	twixt;
+    
+    twixt = thread->thread.twixts;
+    while (twixts--)
+	twixt = twixt->previous;
+    catch = thread->thread.catches;
+    while (catches--)
+	catch = catch->previous;
+    
+    /*
+     * Create a continuation that unwinds the twixts
+     * and catches and then ends up executing the next
+     * instruction
+     */
+    continuation = NewContinuation (thread->thread.frame,
+				    *next,
+				    running->thread.stack,
+				    catch, twixt);
+    (void) do_longjmp (next, continuation, Zero);
+    EXIT ();
+}
+
 static void
 ThreadBoxCheck (BoxPtr box, int i, Types *type)
 {
@@ -586,6 +613,7 @@ ThreadStep (Value thread)
     case OpNameRefStore:
 	switch (inst->var.name->symbol.class) {
 	case class_global:
+	case class_const:
 	    box = inst->var.name->global.value;
 	    i = 0;
 	    break;
@@ -860,7 +888,7 @@ ThreadStep (Value thread)
 	    w = Plus (v, One);
 	else
 	    w = Minus (v, One);
-	ThreadAssign (value, w, True);
+	ThreadAssign (value, w, False);
 	value = v;
 	if (inst->base.opCode == OpPreInc || inst->base.opCode == OpPreDec)
 	    value = w;
@@ -949,14 +977,14 @@ ThreadStep (Value thread)
 	default:
 	    break;
 	}
-	ThreadAssign (value, v, True);
+	ThreadAssign (value, v, False);
 	value = v;
 	break;
     case OpInitialize:
 	if (value->value.tag != type_ref)
 	    break;
 	v = Stack(stack); stack++;
-	ThreadAssign (value, v, False);
+	ThreadAssign (value, v, True);
 	value = v;
 	break;
     case OpEq:
@@ -1036,6 +1064,12 @@ ThreadStep (Value thread)
 	else
 	    next = inst + inst->branch.offset;
 	break;
+    case OpUnwind:
+	if (aborting)
+	    break;
+	ThreadUnwind (thread, inst->unwind.twixt, inst->unwind.catch, &next);
+	complete = True;
+	break;
     }
     if (!aborting || complete)
     {
@@ -1117,7 +1151,12 @@ ThreadsRun (Value thread, Value lex)
 		signalIo = False;
 		IoInterrupt ();
 	    }
-	    if (lex && !(lex->file.flags & FileInputBlocked))
+	    if (signalProfile)
+	    {
+		signalProfile = False;
+		ProfileInterrupt (running);
+	    }
+	    if (lex && !(lex->file.flags & (FileInputBlocked|FileOutputBlocked)))
 		break;
 	}
 	else if (thread && !Runnable (thread))
