@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/float.c,v 1.13 2001/04/09 06:37:43 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/float.c,v 1.15 2001/10/05 07:56:41 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -739,6 +739,7 @@ FloatPrint (Value f, Value fv, char format, int base, int width, int prec, unsig
     Value	int_part;
     Value	frac_part;
     unsigned	length;
+    int		orig_prec = prec;
     int		mant_prec;
     int		frac_prec;
     int		dig_max;
@@ -777,20 +778,17 @@ FloatPrint (Value f, Value fv, char format, int base, int width, int prec, unsig
 	return False;
     }
     DebugV ("expbase", expbase);
-    DebugV ("ratio  ", ratio);
+    DebugF ("ratio  ", &ratio->floats);
     down = Pow (NewInt (2),
 		NewInt ((int) length));
     DebugV ("down   ", down);
     fratio = Divide (ratio, down);
-    DebugV ("fratio ", fratio);
+    DebugF ("fratio ", &fratio->floats);
     negative = a->mant->sign == Negative;
     m = NewInteger (Positive, a->mant->mag);
     if (FpartLength (a->mant) == a->prec)
 	m = Plus (m, One);
     m = Times (m, fratio);
-    if (prec > 0)
-    {
-    }
     if (True (Less (m, One)))
     {
 	m = Times (m, NewInt (base));
@@ -874,15 +872,29 @@ FloatPrint (Value f, Value fv, char format, int base, int width, int prec, unsig
 	int_width++;
     }
     
-    frac_width = prec - int_width - exp_width;
-    
-    if (frac_prec + 1 < frac_width || prec == INFINITE_OUTPUT_PRECISION)
-	frac_width = frac_prec + 1;
+    if (width)
+    {
+	if (width > 0)
+	    frac_width = width - int_width - exp_width;
+	else
+	    frac_width = -width - int_width - exp_width;
+	if (prec > 0)
+	    if (frac_width > prec + 1)
+		frac_width = prec + 1;
+    }
+    else
+    {
+	if (prec == INFINITE_OUTPUT_PRECISION)
+	    frac_width = frac_prec + 1;
+	else
+	    frac_width = prec + 1;
+    }
+
     if (frac_width < 2)
 	frac_width = 0;
     frac_buffer = 0;
     frac_string = 0;
-    if (frac_width && !Zerop (frac_part))
+    if (frac_width && (!Zerop (frac_part) || orig_prec > 0))
     {
 	int	frac_wrote;
 	
@@ -903,6 +915,8 @@ FloatPrint (Value f, Value fv, char format, int base, int width, int prec, unsig
 	}
 	*--frac_string = '.';
     }
+    else
+	frac_width = 0;
 
     print_width = int_width + frac_width + exp_width;
     while (width > print_width)
@@ -922,6 +936,11 @@ FloatPrint (Value f, Value fv, char format, int base, int width, int prec, unsig
 	    FilePuts (f, "-");
 	FilePuts (f, exp_string);
     }
+    while (-width > print_width)
+    {
+	FileOutput (f, fill);
+	width++;
+    }
     free (int_buffer);
     if (frac_buffer)
 	free (frac_buffer);
