@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/frame.c,v 1.9 2001/04/09 03:20:02 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/frame.c,v 1.10 2001/08/14 08:34:13 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -41,7 +41,7 @@ NewFrame (Value		function,
     frame->staticLink = staticLink;
     frame->function = function;
     if (dynamics)
-	frame->frame = NewTypedBox (False, False, dynamics);
+	frame->frame = NewTypedBox (False, dynamics);
     frame->statics = statics;
     RETURN (frame);
 }
