@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/gcd.c,v 1.12 2001/03/30 23:30:39 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/gcd.c,v 1.13 2001/07/03 04:08:31 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -137,6 +137,7 @@ DigitBmod (digit u, digit v, int s)
 
 static int
 DigitWidth (digit i)
+#if DIGITBITS == 32
 {
     int	    w;
 
@@ -161,6 +162,29 @@ DigitWidth (digit i)
 	w++;
     return w;
 }
+#endif
+#if DIGITBITS == 16
+{
+    int	    w;
+
+    w = 0;
+    if (i & 0xff00)
+	w += 8;
+    else
+	i <<= 8;
+    if (i & 0xf000)
+	w += 4;
+    else
+	i <<= 4;
+    if (i & 0xc000)
+	w += 2;
+    else
+	i <<= 2;
+    if (i & 0x8000)
+	w++;
+    return w;
+}
+#endif
 
 int
 NaturalWidth (Natural *u)
