@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/nickle.h,v 1.61 2001/04/09 05:45:33 bart Exp $ */
+/* $Header: /local/src/CVS/nickle/nickle.h,v 1.66 2001/08/14 08:34:14 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -58,6 +58,7 @@ typedef union _symbol {
 
 extern SymbolPtr    NewSymbolType (Atom name, Types *type);
 extern SymbolPtr    NewSymbolException (Atom name, Types *type);
+extern SymbolPtr    NewSymbolConst (Atom name, Types *type);
 extern SymbolPtr    NewSymbolGlobal (Atom name, Types *type);
 extern SymbolPtr    NewSymbolArg (Atom name, Types *type);
 extern SymbolPtr    NewSymbolStatic (Atom name, Types *Type);
@@ -198,6 +199,7 @@ typedef struct _exprBase {
     int		    line;
     NamespacePtr    namespace;
     Types	    *type;
+    unsigned long   ticks;
 } ExprBase;
 
 typedef struct _exprTree {
@@ -426,6 +428,12 @@ typedef struct _instTagCase {
     Atom	tag;
 } InstTagCase;
 
+typedef struct _instUnwind {
+    InstBase	inst;
+    int		twixt;
+    int		catch;
+} InstUnwind;
+
 typedef union _inst {
     InstBase	base;
     InstVar	var;
@@ -441,17 +449,36 @@ typedef union _inst {
     InstRaise	raise;
     InstTwixt	twixt;
     InstTagCase	tagcase;
+    InstUnwind	unwind;
 } Inst;
 
-#define OBJ_STATE_ERROR	    1
-#define OBJ_STATE_LOOP	    2
-#define OBJ_STATE_SWITCH    4
+/*
+ * A structure used to process non-structured
+ * control flow changes (break, continue, return)
+ * within the presence of twixt/catch
+ */
+
+typedef enum _nonLocalKind { 
+    NonLocalControl, NonLocalTwixt, NonLocalCatch,
+} NonLocalKind;
+
+#define NON_LOCAL_RETURN    0
+#define NON_LOCAL_BREAK	    1
+#define NON_LOCAL_CONTINUE  2
+
+typedef struct _nonLocal {
+    DataType		*data;
+    struct _nonLocal	*prev;
+    NonLocalKind	kind;	/* kind of non local object */
+    int			target;	/* what kind of targets */
+} NonLocal;
 
 typedef struct _obj {
     DataType	*data;
     int		size;
     int		used;
-    int		state;
+    Bool	error;
+    NonLocal	*nonLocal;
 } Obj;
 
 #define ObjCode(obj,i)	(((InstPtr) (obj + 1)) + (i))
@@ -548,6 +575,8 @@ void	HistoryDisplay (Value, Value *, Value *);
 void	HistoryInsert (Value);
 void	HistoryInit (void);
 
+void	ProfileInterrupt (Value thread);
+
 typedef Bool	(*TimerFunc) (void *closure);
 
 unsigned long	TimeInMs (void);
@@ -617,8 +646,6 @@ extern int	stdin_interactive;
 void	intr(int);
 void	stop (int), die (int), segv (int);
 
-#define HAS_SIGACTION
-
 void
 catchSignal (int sig, RETSIGTYPE (*func) (int sig));
 
@@ -746,6 +773,7 @@ Value	do_Command_edit (Value);
 Value	do_Environ_check (Value);
 Value	do_Environ_get (Value);
 Value	do_Environ_unset (Value);
+Value	do_profile (Value);
 
 /* two argument builtins */
 Value	do_Thread_set_priority (Value, Value);
@@ -770,6 +798,7 @@ Value	do_Gcd_kary_reduction (Value, Value);
 Value	do_Environ_set (Value, Value);
 
 /* three argument builtins */
+Value	do_File_vfprintf (Value, Value, Value);
 Value	do_String_substr (Value, Value, Value);
 Value	do_File_pipe (Value, Value, Value);
 
