@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/pretty.c,v 1.38 2001/04/08 21:52:48 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/pretty.c,v 1.42 2001/08/14 08:34:14 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -16,6 +16,33 @@
 #include	"nickle.h"
 #include	"gram.h"
 
+extern Bool profiling;
+
+static void
+PrettyProf (Value f, Expr *e)
+{
+    if (profiling)
+    {
+	unsigned long t;
+	int	i,n;
+
+	t = 1;
+	i = 1;
+	while (t < e->base.ticks)
+	{
+	    t *= 10;
+	    i++;
+	}
+	n = 6 - i;
+	if (n < 0)
+	    n = 0;
+	while (n--)
+	    FilePuts (f, " ");
+	FilePrintf (f, "%d: ", e->base.ticks);
+	e->base.ticks = 0;
+    }
+}
+	    
 static void PrettyParameters (Value f, Expr *e, Bool nest);
 static void PrettyArrayInit (Value f, Expr *e, int level, Bool nest);
 static void PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest);
@@ -99,11 +126,13 @@ tokenToPrecedence (int token)
 static void
 PrettyParameters (Value f, Expr *e, Bool nest)
 {
-    if (e->tree.right) {
-	PrettyParameters (f, e->tree.right, nest);
-	FilePuts (f, ", ");
+    while (e) 
+    {
+	PrettyExpr (f, e->tree.left, -1, 0, nest);
+	e = e->tree.right;
+	if (e)
+	    FilePuts (f, ", ");
     }
-    PrettyExpr (f, e->tree.left, -1, 0, nest);
 }
 
 static void
@@ -196,8 +225,14 @@ PrettyDecl (Value f, Expr *e, int level, Bool nest)
 {
     DeclListPtr	decl;
 
+/*    PrettyProf (f, e);*/
     FilePutPublish (f, e->decl.publish, True);
     switch (e->decl.class) {
+    case class_const:
+	FilePuts (f, "const");
+	if (e->decl.type)
+	    FilePutTypes (f, e->decl.type, False);
+	break;
     case class_global:
 	if (e->decl.type)
 	    FilePutTypes (f, e->decl.type, False);
@@ -250,6 +285,7 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest)
 
     if (!e)
 	return;
+/*    PrettyProf (f, e); */
     selfPrec = tokenToPrecedence (e->base.tag);
     if (selfPrec < parentPrec)
 	FilePuts (f, "(");
@@ -292,8 +328,13 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest)
 	FilePuts (f, " }");
 	break;
     case UNION:
-	FilePrintf (f, "(%T.%A) ", e->base.type, e->tree.left->atom.atom);
-	PrettyExpr (f, e->tree.right, selfPrec, level, nest);
+	FilePrintf (f, "%T.%A ", e->base.type, e->tree.left->atom.atom);
+	if (e->tree.right)
+	{
+	    FilePuts (f, "(");
+	    PrettyExpr (f, e->tree.right, selfPrec, level, nest);
+	    FilePuts (f, ")");
+	}
 	break;
     case TEN_CONST:
     case OCTAL_CONST:
@@ -303,6 +344,7 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest)
     case STRING_CONST:
     case POLY_CONST:
     case THREAD_CONST:
+    case VOIDVAL:
 	FilePrintf (f, "%v", e->constant.constant);
 	break;
     case CHAR_CONST:
@@ -447,6 +489,7 @@ PrettyExpr (Value f, Expr *e, int parentPrec, int level, Bool nest)
 void
 PrettyStatement (Value f, Expr *e, int level, int blevel, Bool nest)
 {
+    PrettyProf (f, e);
     switch (e->base.tag) {
     case EXPR:
 	PrettyIndent (f, level);
@@ -731,6 +774,7 @@ doPrettyPrint (Value f, Publish publish, SymbolPtr symbol, int level, Bool nest)
 	return;
     PrettyIndent (f, level);
     switch (symbol->symbol.class) {
+    case class_const:
     case class_global:
 	v = BoxValueGet (symbol->global.value, 0);
 	if (v && v->value.tag == type_func)
