@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/rational.c,v 1.12 2001/04/07 00:02:15 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/rational.c,v 1.16 2001/10/05 07:56:41 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -528,90 +528,50 @@ NewFactor (Natural *prime, int power, FactorPtr next)
     RETURN (f);
 }
 
-static Natural *
-NaturalMod (Natural *n, Natural *m)
+static FactorPtr 
+GenerateFactors (Natural *n, Natural *max)
 {
     ENTER ();
-    Natural *rem;
+    FactorPtr	f = 0;
+    Natural	*p;
+    Natural	*largest;
+    Natural	*d, *rem;
 
-    (void) NaturalDivide (n, m, &rem);
-    RETURN (rem);
-}
-
-#define evenp(n)    ((NaturalDigits(n)[0] & 1) == 0)
-
-static Bool
-IsPrime (Natural *n)
-{
-    Natural *lim;
-    Natural *v;
-
-    if (evenp (n))
-	return 0;
-    lim = NaturalSqrt (n);
-    for (v = NewNatural (3); 
-	 !NaturalLess (lim, v); 
-	 v = NaturalPlus (v, two_natural))
+    p = 0;
+    largest = NaturalSqrt (n);
+    while (!NaturalEqual (n, one_natural))
     {
-	if (NaturalZero (NaturalMod (n, v)))
-	    return 0;
-    }
-    return 1;
-}
-
-static Natural *
-NextPrime (Natural *n)
-{
-    if (NaturalEqual (n, two_natural))
-	return NewNatural (3);
-    do
-    {
-	n = NaturalPlus (n, two_natural);
-    } while (!IsPrime (n));
-    return n;
-}
-
-static FactorPtr
-GenerateFactors (Natural *n, Natural *p, Natural *max)
-{
-    ENTER ();
-    Natural	*rem, *d;
-    int		power;
-    FactorPtr	factors;
-
-    if (NaturalEqual (n, one_natural))
-	RETURN(0);
-    power = 0;
-    for (;;)
-    {
-	d = NaturalDivide (n, p, &rem);
-	if (aborting)
-	    RETURN (0);
-	if (NaturalZero (rem))
+        int power = 1;
+	for (;;)
 	{
-	    power++;
-	    n = d;
-	}
-	else
-	{
-	    if (power)
-	    {
-		factors = NewFactor (p, power, 
-				     GenerateFactors (n, NextPrime (p), max));
-		break;
-	    }
+	    if (!p)
+		p = two_natural;
+	    else if (NaturalEqual (p, two_natural))
+		p = NewNatural (3);
 	    else
-	    {
-		p = NextPrime (p);
-		if (max && NaturalLess (max, p))
-		{
-		    factors = 0;
-		    break;
-		}
-	    }
+		p = NaturalPlus (p, two_natural);
+	    
+	    d = NaturalDivide (n, p, &rem);
+	    if (NaturalZero (rem))
+		break;
+	    if (max && NaturalLess (max, p))
+		RETURN(f);
+	    if (NaturalLess (largest, p))
+		RETURN (NewFactor (n, 1, f));
 	}
+	n = d;
+	for (;;)
+	{
+	    d = NaturalDivide (n, p, &rem);
+	    if (!NaturalZero (rem))
+		break;
+	    n = d;
+	    power++;
+	}
+	f = NewFactor (p, power, f);
+	largest = NaturalSqrt (n);
     }
-    RETURN (factors);
+    RETURN (f);
 }
 
 static Natural *
@@ -689,7 +649,7 @@ RationalRepeatLength (int prec, Natural *nden, int ibase)
     }
     else
     {
-	factors = GenerateFactors (ndigits, two_natural, max);
+	factors = GenerateFactors (ndigits, max);
 	if (aborting)
 	    return 0;
 	factor = one_natural;
@@ -777,7 +737,8 @@ RationalDecimalPrint (Value f, Value rv, char format, int base, int width, int p
     char	*repeat = 0, *re;
     char	*whole;
     int		initial_width, repeat_width;
-    int		rep_width;
+    int		frac_width;
+    int		rep_width, brace_width = 0, dot_width = 0;
     int		whole_width;
     int		fraction_width;
     int		print_width;
@@ -896,6 +857,67 @@ RationalDecimalPrint (Value f, Value rv, char format, int base, int width, int p
     }
     quo = NaturalDivide (r->num, r->den, &partial);
     whole = NaturalSprint (0, quo, base, &whole_width);
+    brace_width = 0;
+    if (repeat_width)
+    {
+	brace_width++;
+	if (repeat_width > 0)
+	    brace_width++;
+    }
+    dot_width = 0;
+    if (initial_width + rep_width)
+	dot_width = 1;
+    /*
+     * Compute how much space is available for the fractional part
+     */
+    if (width)
+    {
+	if (width < 0)
+	    fraction_width = -width;
+	else
+	    fraction_width = width;
+	fraction_width = fraction_width - (whole_width + exponent_width);
+	if (fraction_width < 0)
+	    fraction_width = 0;
+	if (prec > 0 && fraction_width > prec + dot_width)
+	    fraction_width = prec + dot_width;
+    }
+    else if (prec > 0)
+	fraction_width = prec + dot_width;
+    else
+	fraction_width = -1;
+    /*
+     * Start paring down parts of the output to fit the desired size
+     */
+    while (fraction_width >= 0 &&
+	   (frac_width = dot_width + initial_width + rep_width + brace_width)
+	   && frac_width > fraction_width)
+    {
+	if (rep_width)
+	{
+	    if (brace_width > 1)
+	    {
+		brace_width = 1;
+		repeat_width = -repeat_width;
+	    }
+	    rep_width = fraction_width - (dot_width + initial_width +
+					  brace_width);
+	    if (rep_width < 0)
+	    {
+		rep_width = 0;
+	    }
+	}
+	else if (brace_width)
+	    brace_width = 0;
+	else if (initial_width)
+	{
+	    initial_width = fraction_width - dot_width;
+	    if (initial_width < 0)
+		initial_width = 0;
+	}
+	else
+	    dot_width = 0;
+    }
     if (initial_width)
     {
 	init = NaturalDivide (NaturalTimes (partial,
@@ -924,7 +946,7 @@ RationalDecimalPrint (Value f, Value rv, char format, int base, int width, int p
     }
     if (rep_width)
     {
-#define MAX_SENSIBLE	1000000
+#define MAX_SENSIBLE	10000000
 	if (rep_width > MAX_SENSIBLE)
 	{
 	    repeat_width = -1;
