@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/sched.c,v 1.26 2001/04/08 20:12:44 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/sched.c,v 1.28 2001/04/18 15:50:05 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -368,7 +368,7 @@ TraceFrame (FramePtr frame, InstPtr pc)
     EXIT ();
 }
 
-#ifdef DEBUG_JUMPS
+#ifdef DEBUG_JUMP
 static void
 TraceIndent (int indent)
 {
@@ -397,7 +397,7 @@ TraceContinuation (char	    *where,
     {
 	if (s)
 	    FilePuts (FileStdout, ", ");
-	print (FileStdout, STACK_ELT(stack, s));
+	FilePrintf (FileStdout, "%v", STACK_ELT(stack, s));
     }
     FilePuts (FileStdout, "\n");
     TraceIndent (indent);
@@ -406,12 +406,12 @@ TraceContinuation (char	    *where,
     {
 	if (s)
 	    FilePuts (FileStdout, ", ");
-	FilePuts (FileStdout, AtomName (catches->exception->symbol.name));
+	FilePrintf (FileStdout, "%A", catches->exception->symbol.name);
     }
     FilePuts (FileStdout, "\n");
     TraceIndent (indent);
     FilePuts (FileStdout, "statement: ");
-    PrintStat (FileStdout, pc->base.stat, False);
+    PrettyStat (FileStdout, pc->base.stat, False);
     for (s = 0; twixts; twixts = twixts->previous, s++)
     {
 	TraceContinuation ("twixt",
@@ -526,7 +526,7 @@ NewThread (FramePtr frame, ObjPtr code)
     ret->thread.id = ++ThreadId;
     ret->thread.partial = 0;
     complete = True;
-    if (code->state & OBJ_STATE_ERROR)
+    if (code->error)
 	ret->thread.state = ThreadFinished;
     _ThreadInsert (ret);
     RETURN (ret);
