@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/type.c,v 1.39 2001/04/08 20:12:44 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/type.c,v 1.42 2001/08/14 08:34:14 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -171,11 +171,12 @@ NewTypesStruct (StructType *structs)
     t = ALLOCATE (&TypesStructType, sizeof (TypesStruct));
     t->base.tag = types_struct;
     t->structs.structs = structs;
+    t->structs.enumeration = False;
     RETURN (t);
 }
 
 Types *
-NewTypesUnion (StructType *structs)
+NewTypesUnion (StructType *structs, Bool enumeration)
 {
     ENTER ();
     Types   *t;
@@ -183,6 +184,7 @@ NewTypesUnion (StructType *structs)
     t = ALLOCATE (&TypesStructType, sizeof (TypesStruct));
     t->base.tag = types_union;
     t->structs.structs = structs;
+    t->structs.enumeration = enumeration;
     RETURN (t);
 }
 
@@ -369,7 +371,10 @@ TypeCompatible (Types *a, Types *b, Bool contains)
 		return True;
     }
     
-    if (TypePoly (a) || TypePoly (b))
+    if (TypePoly (a))
+	return True;
+
+    if (/* !contains && */ TypePoly (b))
 	return True;
 
     if (a->base.tag != b->base.tag)
@@ -736,7 +741,7 @@ TypeCombineFlatten (Types **rets, int nret, int sret)
     if (nret == 1)
 	RETURN (rets[0]);
     
-    ret = NewTypesUnion (NewStructType (nret));
+    ret = NewTypesUnion (NewStructType (nret), False);
     st = ret->structs.structs;
     for (nret = 0; nret < st->nelements; nret++)
 	StructTypeElements(st)[nret].type = rets[nret];
@@ -874,8 +879,10 @@ TypeCombineBinary (Types *left, int tag, Types *right)
 	break;
     case EQ:
     case NE:
+#if 0
     	rets = TypeAdd (rets, typesPrim[type_integer], &nret, &sret);
 	break;
+#endif
     case LT:
     case GT:
     case LE:
@@ -936,7 +943,7 @@ TypeCombineUnary (Types *type, int tag)
 	    rets = TypeAdd (rets, ret, &nret, &sret);
 	break;
     case BANG:
-	ret = TypeCombineBinary (type, EQ, typesPrim[type_integer]);
+	ret = TypeCombineBinary (type, EQ, typesNil);
 	if (ret)
 	    rets = TypeAdd (rets, ret, &nret, &sret);
 	break;
@@ -1103,7 +1110,12 @@ TypeCompatibleAssign (TypesPtr a, Value b)
 	return TypeCompatibleAssign (a->name.type, b);
     case types_ref:
 	if (b->value.tag == type_ref)
-	    return TypeCompatible (a->ref.ref, RefType (b), True);
+	{
+	    if (RefValueGet (b))
+		return TypeCompatibleAssign (a->ref.ref, RefValueGet (b));
+	    else
+		return TypeCompatible (a->ref.ref, RefType (b), True);
+	}
 	if (b->value.tag == type_int && b->ints.value == 0)
 	    return True;
 	break;
@@ -1136,7 +1148,26 @@ TypeCompatibleAssign (TypesPtr a, Value b)
 	    adim = TypeCountDimensions (a->array.dimensions);
 	    bdim = b->array.ndim;
 	    if (adim == 0 || adim == bdim)
-		return TypeCompatible (a->array.type, b->array.type, True);
+	    {
+		if (TypePoly (a->array.type))
+		    return True;
+		if (TypePoly (b->array.type))
+		{
+		    int	i;
+		    BoxPtr  box = b->array.values;
+
+		    for (i = 0; i < b->array.ents; i++)
+			if (BoxValueGet (box, i) &&
+			    !TypeCompatibleAssign (a->array.type,
+						   BoxValueGet (box, i)))
+			{
+			    return False;
+			}
+		    return True;
+		}
+		else
+		    return TypeCompatible (a->array.type, b->array.type, True);
+	    }
 	}
 	break;
     case types_struct:
@@ -1195,19 +1226,19 @@ TypesInit (void)
     StructTypeElements(st)[0].type = typesPrim[type_integer];
     StructTypeElements(st)[1].type = typesPrim[type_rational];
     StructTypeElements(st)[2].type = typesPrim[type_float];
-    typesGroup = NewTypesUnion (st);
+    typesGroup = NewTypesUnion (st, False);
     MemAddRoot (typesGroup);
     
     st = NewStructType (2);
     StructTypeElements(st)[0].type = typesPrim[type_rational];
     StructTypeElements(st)[1].type = typesPrim[type_float];
-    typesField = NewTypesUnion (st);
+    typesField = NewTypesUnion (st, False);
     MemAddRoot (typesField);
     
     st = NewStructType (2);
     StructTypeElements(st)[0].type = typesPrim[type_int];
     StructTypeElements(st)[1].type = typesRefPoly;
-    typesNil = NewTypesUnion (st);
+    typesNil = NewTypesUnion (st, False);
     MemAddRoot (typesField);
     
     TypeCheckStack = StackCreate ();
