@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/value.h,v 1.57 2001/04/08 21:52:48 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/value.h,v 1.60 2001/08/14 08:34:14 keithp Exp $ */
 
 /*
  * Copyright (C) 1988-2001 Keith Packard and Bart Massey.
@@ -63,32 +63,92 @@ AtomListPtr  NewAtomList (AtomListPtr next, Atom atom);
  * still work correctly.
  */
 
-#if SIZEOF_LONG_LONG == 8
-# define BASE		((double_digit) 65536 * (double_digit) 65536)
-# define LBASE2	32
-# define LLBASE2	5
-# define DIGITBITS	32
+#if HAVE_STDINT_H
+
+# include <stdint.h>
+# if HAVE_UINT64_T
+
+/*
+ * If stdint.h defines a 64 bit datatype, use 32 bit
+ * chunks
+ */
+
+#  define DIGITBITS 32
+typedef uint64_t    double_digit;
+typedef int64_t	    signed_digit;
+typedef uint32_t    digit;
+
+# else
+
+#  define DIGITBITS 16
+typedef uint32_t    double_digit;
+typedef uint16_t    digit;
+typedef int32_t	    signed_digit;
+
+# endif
+
 #else
-# define BASE		((double_digit) 65536)
-# define LBASE2	16
-# define LLBASE2	4
-# define DIGITBITS	16
+
+# if SIZEOF_UNSIGNED_LONG_LONG == 8 || SIZEOF_UNSIGNED_LONG == 8
+#  define DIGITBITS 32
+# else
+#  define DIGITBITS 16
+# endif
+
+# if DIGITBITS == 32
+
+#  if SIZEOF_UNSIGNED_LONG_LONG == 8
+typedef unsigned long long double_digit;
+typedef long long signed_digit;
+#  else
+#   if SIZEOF_UNSIGNED_LONG == 8
+typedef unsigned long double_digit;
+typedef long signed_digit;
+#   endif
+#  endif
+
+#  if SIZEOF_UNSIGNED_LONG == 4
+typedef unsigned long digit;
+#  else
+#   if SIZEOF_UNSIGNED_INT == 4
+typedef unsigned int digit;
+#   endif
+#  endif
+
+# else
+
+#  if SIZEOF_UNSIGNED_LONG == 4
+typedef unsigned long double_digit;
+typedef long signed_digit;
+#  else
+#   if SIZEOF_UNSIGNED_INT == 4
+typedef unsigned int double_digit;
+typedef int signed_digit;
+#   endif
+#  endif
+
+#  if SIZEOF_UNSIGNED_INT == 2
+typedef unsigned int digit;
+#  else
+#   if SIZEOF_UNSIGNED_SHORT == 2
+typedef unsigned short digit;
+#   endif
+#  endif
+
+# endif
+
 #endif
 
 #define MAXDIGIT	((digit) (BASE - 1))
 
-/*
- * Natural numbers form the basis for both the Integers and Rationals,
- * but needn't ever be exposed to the user
- */
-#if DIGITBITS <= 16
-typedef unsigned short	digit;		/* must hold 0 .. BASE - 1 */
-typedef unsigned int	double_digit;	/* must hold 0 .. (BASE - 1) * (BASE - 1) + BASE-2 */
-typedef int		signed_digit;	/* must hold -BASE .. BASE - 1 */
+#if DIGITBITS == 32
+# define BASE		((double_digit) 65536 * (double_digit) 65536)
+# define LBASE2	32
+# define LLBASE2	5
 #else
-typedef unsigned int	    digit;
-typedef unsigned long long  double_digit;
-typedef signed long long    signed_digit;
+# define BASE		((double_digit) 65536)
+# define LBASE2	16
+# define LLBASE2	4
 #endif
 
 #define TwoDigits(n,i)	((double_digit) NaturalDigits(n)[i-1] | \
@@ -96,6 +156,11 @@ typedef signed long long    signed_digit;
 #define ModBase(t)  ((t) & (((double_digit) 1 << LBASE2) - 1))
 #define DivBase(t)  ((t) >> LBASE2)
     
+/*
+ * Natural numbers form the basis for both the Integers and Rationals,
+ * but needn't ever be exposed to the user
+ */
+
 typedef struct _natural {
     DataType	*type;
     int		length;
@@ -203,7 +268,7 @@ typedef struct _argType {
     struct _argType *next;
 } ArgType;
 
-ArgType *NewArgType (TypesPtr type, Bool varargs, Atom name, 
+ArgType *NewArgType (TypesPtr type, Bool varargs, Atom name,
 		     SymbolPtr symbol, ArgType *next);
 
 typedef enum _typesTag {
@@ -247,6 +312,7 @@ typedef struct _typesArray {
 typedef struct _typesStruct {
     TypesBase	    base;
     StructTypePtr   structs;
+    Bool	    enumeration;
 } TypesStruct;    
 
 typedef union _types {
@@ -276,12 +342,14 @@ extern Types	    *typesRefPoly;
 extern Types	    *typesNil;
 extern Types	    *typesPrim[type_void - type_int + 1];
 
+#define typesEnum   ((Types *) 1)
+
 Types	*NewTypesName (ExprPtr expr, Types *type);
 Types	*NewTypesRef (Types *ref);
 Types	*NewTypesFunc (Types *ret, ArgType *args);
 Types	*NewTypesArray (Types *type, ExprPtr dimensions);
 Types	*NewTypesStruct (StructTypePtr structs);
-Types	*NewTypesUnion (StructTypePtr structs);
+Types	*NewTypesUnion (StructTypePtr structs, Bool enumeration);
 Types	*TypesCanon (Types *type);
 Type	BaseType (Types *type);
 int	TypesInit (void);
@@ -309,13 +377,14 @@ int	TypeCountDimensions (ExprPtr dims);
  */
 
 typedef enum _class {
-    class_global, class_static, class_arg, class_auto, 
+    class_global, class_static, class_arg, class_auto, class_const,
     class_typedef, class_namespace, class_exception, class_undef
 } Class;
 
 #define ClassLocal(c)	((c) == class_arg || (c) == class_auto)
 #define ClassFrame(c)	((c) == class_static || ClassLocal(c))
-#define ClassStorage(c)	((c) <= class_auto)
+#define ClassStorage(c)	((c) <= class_const)
+#define ClassLvalue(c)	((c) <= class_auto)
 
 typedef enum _publish {
     publish_private, publish_protected, publish_public, publish_extend
@@ -420,7 +489,7 @@ typedef struct _ref {
 #define RefValue(r)	BoxValue((r)->ref.box, (r)->ref.element)
 #define RefValueGet(r)	BoxValueGet((r)->ref.box, (r)->ref.element)
 #define RefType(r)	BoxType((r)->ref.box, (r)->ref.element)
-#define RefConstant(r)    BoxConstant((r)->ref.box)
+#define RefConstant(r)    BoxConstant((r)->ref.box, (r)->ref.element)
 
 typedef struct _structElement {
     TypesPtr	type;
@@ -572,7 +641,7 @@ typedef struct _box {
 #define BoxElements(box)	((BoxElement *) ((box) + 1))
 #define BoxValueSet(box,e,v)	((BoxElements(box)[e].value) = (v))
 #define BoxValueGet(box,e)	((BoxElements(box)[e].value))
-#define BoxConstant(box)	((box)->constant)
+#define BoxConstant(box,e)	((box)->constant)
 #define BoxType(box,e)		(BoxElements(box)[e].type)
 
 extern BoxPtr	NewBox (Bool constant, Bool array, int nvalues);
@@ -591,7 +660,7 @@ extern BoxTypesPtr NewBoxTypes (int size);
 
 extern int	AddBoxTypes (BoxTypesPtr *btp, TypesPtr t);
 
-extern BoxPtr	NewTypedBox (Bool constant, Bool array, BoxTypesPtr types);
+extern BoxPtr	NewTypedBox (Bool array, BoxTypesPtr types);
 			     
 Value	NewInt (int value);
 Value	NewInteger (Sign sign, Natural *mag);
@@ -728,10 +797,12 @@ extern volatile Bool signaling;	/* some signal is pending */
 extern volatile Bool signalInterrupt;	/* keyboard interrupt */
 extern volatile Bool signalTimer;	/* timer interrupt */
 extern volatile Bool signalIo;		/* i/o interrupt */
+extern volatile Bool signalProfile;	/* vtimer interrupt */
 
 #define SetSignalInterrupt()(aborting = signaling = signalInterrupt = True)
 #define SetSignalTimer()    (signaling = signalTimer = True)
 #define SetSignalIo()	    (signaling = signalIo = True)
+#define SetSignalProfile()  (signaling = signalProfile = True)
 
 /*
  * Any signal state set by regular code doesn't need to be volatile
